#include <arpa/inet.h>
#include <assert.h>
#include <errno.h>
#include <error.h>
#include <fcntl.h>
#include <getopt.h>
#include <immintrin.h>
#include <inttypes.h>
#include <limits.h>
#include <linux/if_packet.h>
#include <math.h>
#include <netdb.h>
#include <netinet/ether.h>
#include <netinet/in.h>
#include <net/if.h>
#include <pthread.h>
#include <sched.h>
#include <semaphore.h>
#include <signal.h>
#include <stdarg.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>
#include "../utils.c"

#define AVX2

typedef struct {
  float re;
  float im;
} Complex;

#include "../bf1_avx2.c"
#include "../bf1_sse4.c"

#define N_CHANNELS 4
#define N_SAMPLES 32
#define BASIC_FRAME_FREQ 3840000

char in[256];
char out[256];
char out2[256];

int main(int argc, char ** argv) {

  int compress;
  int avx;
  int16_t int16_samples[64];
  const float mult = 1. / 32767.;

  if(argc != 2)
    fprintf(stderr, "Wrong argument number");

  compress = (argv[1][0] == 'c');
  avx = (argv[1][1] == 'a');

  freopen(NULL, "rb", stdin);
  fread(in, compress ? (64 * 4) : 60 , 1, stdin);
  fclose(stdin);

  if(compress) {
    if(avx) {
      Complex ** iq_samples;
      iq_samples = (Complex **) malloc(sizeof(Complex *));
      iq_samples[0] = (Complex *) in;
      encode_data_compressed_16_32_4(1, (uint8_t *) out, 1, iq_samples, (uint8_t *) out);
    } else {
      float_to_int16(int16_samples, (float *) in, 64, 32767);
      encode_bf1(out, int16_samples);
    }
  } else {
    if(avx) {
      Complex ** iq_samples;
      iq_samples = (Complex **) malloc(sizeof(Complex *));
      iq_samples[0] = (Complex *) out;
      decode_data_compressed_16_32_4(1, iq_samples, 1, (uint8_t *) in, (uint8_t *) in);
    } else {
      decode_bf1(int16_samples, in, 16);
      int16_to_float((float *) out, int16_samples, 64, mult);
    }
  }

  freopen(NULL, "wb", stdout);
  fwrite(out, compress ? 60 : (64 * 4), 1, stdout);
  fclose(stdout);
  
  return 0;
}
