#include <arpa/inet.h>
#include <assert.h>
#include <errno.h>
#include <error.h>
#include <fcntl.h>
#include <getopt.h>
#include <immintrin.h>
#include <inttypes.h>
#include <limits.h>
#include <linux/if_packet.h>
#include <math.h>
#include <netdb.h>
#include <netinet/ether.h>
#include <netinet/in.h>
#include <net/if.h>
#include <pthread.h>
#include <sched.h>
#include <semaphore.h>
#include <signal.h>
#include <stdarg.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include "trx-ecpri/trx_ecpri_dpdk.c"

TRXState s1;
float ** tx_samples_zeroes;
float ** tx_samples_ones;
float ** rx_samples;

void dummy_enb_init(TRXState *s1, TRXEcpriState *s);
static void enb(TRXState *s1, TRXEcpriState * s);

int main(int argc, char * argv[]) {
  (void) argc;
  (void) argv;
  TRXEcpriState *s;

  // Lock all current and future pages from preventing of being paged to
  // swap
  if (mlockall(MCL_CURRENT | MCL_FUTURE)) {
    log_error("TRX_ECPRI", "mlockall failed");
  }

  s = malloc(sizeof(TRXEcpriState));
  memset(s, 0, sizeof(*s));

#if 0
  // tiogapass-003 MT27710
  s->rec_mac = "b8:59:9f:07:7e:2a";
  //s->re_mac = "04:09:a5:0f:9f:4c"; // Lille M6424 Switch
  s->re_mac = "b8:59:9f:07:86:42"; // tiogapass-004 MT27710
  //s->re_mac = "b4:96:91:a7:1c:f4"; // tiogapass-004 XXV710DA2T port0
  s->rec_if = "ens9f0";
  s->dpdk_options = "-l 10,20 -b 0000:04:00.0 -b 0000:5e:00.1 ";
#endif

#if 0
  // tiogapass-003 MT27710 port1
  s->rec_mac = "b8:59:9f:07:7e:2b";
  s->re_mac = "04:09:a5:0f:9f:4c"; // Lille M6424 Switch
  //s->re_mac = "b8:59:9f:07:86:42"; // tiogapass-004 MT27710
  //s->re_mac = "b4:96:91:a7:1c:f5"; // tiogapass-004 XXV710DA2T port1
  s->rec_if = "ens9f1";
  s->dpdk_options = "-l 10,20 -b 0000:04:00.0 -b 0000:5e:00.0 ";
#endif

#if 0
  // hfr-tiogapass-001 MT27710
  s->rec_mac = "b8:59:9f:07:82:ca";
  s->re_mac = "04:09:a5:0f:76:1c"; // HFR M6424 switch
  s->rec_if = "ens9f0";
  s->dpdk_options = "-l 10,20 -b 0000:04:00.0 -b 0000:18:00.0 -b 0000:18:00.1 -b 0000:5e:00.1 ";
#endif

#if 0
  // hfr-tiogapass-001 XXV710DA2T
  s->rec_mac = "b4:96:91:a7:1b:28";
  s->re_mac = "04:09:a5:0f:76:1c"; // HFR M6424 switch
  s->rec_if = "ens1f0";
  s->dpdk_options = "-l 10,20 -b 0000:04:00.0 -b 0000:18:00.1 -b 0000:5e:00.0 -b 0000:5e:00.1 ";
#endif

#if 0
  // tiogapass-004 MT27710
  s->rec_mac = "b8:59:9f:07:86:42";
  s->re_mac = "04:09:a5:0f:9f:4a"; // Lille M6424 Switch
  //s->re_mac = "b8:59:9f:07:7e:2a"; // tiogapass-003 MT27710
  s->rec_if = "ens9f0";
  s->dpdk_options = "-l 10,20 -b 0000:04:00.0 -b 0000:3b:00.0 -b 0000:3b:00.1 -b 0000:5e:00.1 ";
#endif

#if 0
  // tiogapass-004 MT27710 port1
  s->rec_mac = "b8:59:9f:07:86:43";
  //s->re_mac = "04:09:a5:0f:9f:4c"; // Lille M6424 Switch
  s->re_mac = "b8:59:9f:07:0e:2b"; // tiogapass-003 MT27710 port 1
  s->rec_if = "ens9f1";
  s->dpdk_options = "-l 10,20 -b 0000:04:00.0 -b 0000:3b:00.0 -b 0000:3b:00.1 -b 0000:5e:00.0 ";
#endif

#if 0
  // tiogapass-004 XXV710DA2T
  s->rec_mac = "b4:96:91:a7:1c:f4";
  s->re_mac = "04:09:a5:0f:9f:4c"; // Lille M6424 Switch
  s->re_mac = "b8:59:9f:07:7e:2a"; // tiogapass-003 MT27710
  s->rec_if = "ens5f0";
  s->dpdk_options = "-l 10,20 -b 0000:04:00.0 -b 0000:3b:00.1 -b 0000:5e:00.0 -b 0000:5e:00.1 ";
#endif

  //s->recv_affinity = 39;
  //s->send_affinity = 38;
  //s->prepare_affinity = 37;
  //s->decompress_affinity = 36;
  //s->statistic_affinity = 35;
  //s->ecpri_period = 800;
  //s->flow_id = 0;
  //s->sample_rate = 122880000;

  //s->trace_file = "/root/ecpri-logs/rx.trace";
  //s->stats_file = "/root/ecpri-logs/ecpri.stats";
  //s->trace_period = 1000000;

  // enb.cfg
  //s->rec_mac = "b8:59:9f:07:86:42";
  //s->re_mac = "04:09:a5:0f:9f:4a";
  //s->rec_if = "ens9f0";
  //s->dpdk_options = "-l 10,20 -b 0000:04:00.0 -b 0000:3b:00.0 -b 0000:3b:00.1 -b 0000:5e:00.1 ";
  s->rec_mac = "b8:59:9f:07:86:42"; /* Mellanox */
  s->re_mac = "b8:59:9f:07:7e:2b"; /* Tiogapass003 */
  s->rec_if = "ens9f0"; /* Mellanox */
  s->dpdk_options = "-l 10,20 -b 0000:04:00.0 -b 0000:3b:00.0 -b 0000:3b:00.1 -b 0000:5e:00.1 "; /* Mellanox */
  s->recv_affinity = 39;
  s->send_affinity = 38;
  s->encode_affinity = 37;
  s->decode_affinity = 36;
  s->statistic_affinity = 35;
  s->ecpri_period = 800;
  s->flow_id = 0;
  s->log_directory = "/root/ecpri-logs";


  log_info("TEST-DPDK-ECPRI", "Starting test...\n");
  log_info("TEST-DPDK-ECPRI", "rec-mac: %s, re-mac: %s, rec-if: %s", s->rec_mac, s->re_mac, s->rec_if);

  dummy_enb_init(&s1, s);
  enb(&s1, s);

}

#define PERIOD_NS INT64_C(1000 * 100)
#define NS_PER_SEC INT64_C(1000000000)
#define FREQ INT64_C(3840000)

static void enb(TRXState * s1, TRXEcpriState * s) {
  struct timespec next;
  trx_timestamp_t ptimestamp;
  int64_t first_rx_ts;

  clock_gettime(CLOCK_TAI, &next);
  tx_samples_zeroes = (float**) malloc(sizeof(float*) * 4);
  tx_samples_ones = (float**) malloc(sizeof(float*) * 4);
  rx_samples = (float**) malloc(sizeof(float*) * 4);
  for(int i = 0; i < 4; i++) {
    tx_samples_zeroes[i] = (float*) malloc(sizeof(float) * 65536);
    tx_samples_ones[i] = (float*) malloc(sizeof(float) * 65536);
    rx_samples[i] = (float*) malloc(sizeof(float) * 65536);
    for(int j = 0; j < 65536; j++) {
      tx_samples_zeroes[i][j] = 0.0f;
      tx_samples_ones[i][j] = 1.0f;
    }
  }

  for(int i = 0;;) {
    for(int j = 0; j < 3; j++) {
      //int64_t tx_timestamp = (32 * FREQ * PERIOD_NS * (i + 1)) / (NS_PER_SEC);
      add_ns(&next, PERIOD_NS);

      s1->trx_read_func2(s1, &ptimestamp, rx_samples, 32, 0, NULL);
      if(i == 0)
        first_rx_ts = ptimestamp;
      s1->trx_write_func2(s1, first_rx_ts + i*32, NULL, 32, 0, NULL);

      i++;
      clock_nanosleep(CLOCK_TAI, TIMER_ABSTIME, &next, NULL);
    }
    for(int j = 0; j < 2; j++) {
      //int64_t tx_timestamp = (32 * FREQ * PERIOD_NS * (i + 1)) / (NS_PER_SEC);
      add_ns(&next, PERIOD_NS);

      if(i == 0)
        first_rx_ts = ptimestamp;
      s1->trx_read_func2(s1, &ptimestamp, rx_samples, 32, 0, NULL);
      s1->trx_write_func2(s1, first_rx_ts + i*32, tx_samples_ones, 32, 0, NULL);

      i++;
      clock_nanosleep(CLOCK_TAI, TIMER_ABSTIME, &next, NULL);
    }
  }
}
